"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReactNativeAskar = void 0;
const askar_shared_1 = require("@openwallet-foundation/askar-shared");
const serialize_1 = require("./serialize");
class ReactNativeAskar {
    constructor(bindings) {
        this.askar = bindings;
    }
    /**
     * Fetch the error from the native library and return it as a JS error
     *
     * NOTE:
     * Checks whether the error code of the returned error matches the error code that was passed to the function.
     * If it doesn't, we throw an error with the original errorCode, and a custom message explaining we weren't able
     * to retrieve the error message from the native library. This should however not break functionality as long as
     * error codes are used rather than error messages for error handling.
     *
     */
    getAskarError(errorCode) {
        const error = this.getCurrentError();
        if (error.code !== errorCode) {
            return new askar_shared_1.AskarError({
                code: errorCode,
                message: 'Error details have already been overwritten on the native side, unable to retrieve error message for the error',
            });
        }
        return new askar_shared_1.AskarError(error);
    }
    handleError({ errorCode, value }) {
        if (errorCode === 0)
            return value;
        throw this.getAskarError(errorCode);
    }
    promisify(method) {
        return new Promise((resolve, reject) => {
            const _cb = ({ errorCode }) => {
                try {
                    this.handleError({ errorCode });
                    resolve();
                }
                catch (e) {
                    reject(e);
                }
            };
            method(_cb);
        });
    }
    promisifyWithResponse(method) {
        return new Promise((resolve, reject) => {
            const _cb = ({ errorCode, value }) => {
                if (errorCode !== 0) {
                    const error = this.getAskarError(errorCode);
                    reject(error);
                }
                else {
                    if (value === undefined) {
                        reject(askar_shared_1.AskarError.customError({ message: 'error code was 0 but no value found. This should not occur.' }));
                    }
                    else {
                        resolve(value);
                    }
                }
            };
            method(_cb);
        });
    }
    version() {
        return (0, askar_shared_1.handleInvalidNullResponse)(this.askar.version({}));
    }
    getCurrentError() {
        const serializedError = (0, askar_shared_1.handleInvalidNullResponse)(this.askar.getCurrentError({}));
        return JSON.parse(serializedError);
    }
    clearCustomLogger() {
        throw new Error('Method not implemented. clearCustomLogger');
    }
    setCustomLogger() {
        throw new Error('Method not implemented. setCustomLogger');
    }
    setDefaultLogger() {
        this.askar.setDefaultLogger({});
    }
    setMaxLogLevel() {
        throw new Error('Method not implemented. setMaxLogLevel');
    }
    entryListCount(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.entryListCount(serializedOptions)));
    }
    entryListFree(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        // null response is expected as we're freeing the object
        this.handleError(this.askar.entryListFree(serializedOptions));
    }
    entryListGetCategory(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.entryListGetCategory(serializedOptions)));
    }
    entryListGetName(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.entryListGetName(serializedOptions)));
    }
    entryListGetTags(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return this.handleError(this.askar.entryListGetTags(serializedOptions));
    }
    entryListGetValue(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const buf = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.entryListGetValue(serializedOptions)));
        return new Uint8Array(buf);
    }
    keyAeadDecrypt(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const buf = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyAeadDecrypt(serializedOptions)));
        return new Uint8Array(buf);
    }
    keyAeadEncrypt(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const ret = this.handleError(this.askar.keyAeadEncrypt(serializedOptions));
        const { buffer, noncePos, tagPos } = (0, askar_shared_1.handleInvalidNullResponse)(ret);
        return new askar_shared_1.EncryptedBuffer({ tagPos, noncePos, buffer: new Uint8Array(buffer) });
    }
    keyAeadGetPadding(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyAeadGetPadding(serializedOptions)));
    }
    keyAeadGetParams(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const ret = this.handleError(this.askar.keyAeadGetParams(serializedOptions));
        const { tagLength, nonceLength } = (0, askar_shared_1.handleInvalidNullResponse)(ret);
        return new askar_shared_1.AeadParams({ nonceLength, tagLength });
    }
    keyAeadRandomNonce(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const buf = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyAeadRandomNonce(serializedOptions)));
        return new Uint8Array(buf);
    }
    keyConvert(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyConvert(serializedOptions)));
        return new askar_shared_1.LocalKeyHandle(handle);
    }
    keyCryptoBox(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const buf = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyCryptoBox(serializedOptions)));
        return new Uint8Array(buf);
    }
    keyCryptoBoxOpen(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const buf = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyCryptoBoxOpen(serializedOptions)));
        return new Uint8Array(buf);
    }
    keyCryptoBoxRandomNonce() {
        const buf = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyCryptoBoxRandomNonce({})));
        return new Uint8Array(buf);
    }
    keyCryptoBoxSeal(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const buf = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyCryptoBoxSeal(serializedOptions)));
        return new Uint8Array(buf);
    }
    keyCryptoBoxSealOpen(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const buf = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyCryptoBoxSealOpen(serializedOptions)));
        return new Uint8Array(buf);
    }
    keyDeriveEcdh1pu(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyDeriveEcdh1pu(serializedOptions)));
        return new askar_shared_1.LocalKeyHandle(handle);
    }
    keyDeriveEcdhEs(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyDeriveEcdhEs(serializedOptions)));
        return new askar_shared_1.LocalKeyHandle(handle);
    }
    keyEntryListCount(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyEntryListCount(serializedOptions)));
    }
    keyEntryListFree(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        // null resopnse is expected as we're freeing the object
        this.handleError(this.askar.keyEntryListFree(serializedOptions));
    }
    keyEntryListGetAlgorithm(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyEntryListGetAlgorithm(serializedOptions)));
    }
    keyEntryListGetMetadata(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return this.handleError(this.askar.keyEntryListGetMetadata(serializedOptions));
    }
    keyEntryListGetName(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyEntryListGetName(serializedOptions)));
    }
    keyEntryListGetTags(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return this.handleError(this.askar.keyEntryListGetTags(serializedOptions));
    }
    keyEntryListLoadLocal(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyEntryListLoadLocal(serializedOptions)));
        return new askar_shared_1.LocalKeyHandle(handle);
    }
    keyFree(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        // null resopnse is expected as we're freeing the object
        this.handleError(this.askar.keyFree(serializedOptions));
    }
    keyFromJwk(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyFromJwk(serializedOptions)));
        return new askar_shared_1.LocalKeyHandle(handle);
    }
    keyFromKeyExchange(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyFromKeyExchange(serializedOptions)));
        return new askar_shared_1.LocalKeyHandle(handle);
    }
    keyFromPublicBytes(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyFromPublicBytes(serializedOptions)));
        return new askar_shared_1.LocalKeyHandle(handle);
    }
    keyFromSecretBytes(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyFromSecretBytes(serializedOptions)));
        return new askar_shared_1.LocalKeyHandle(handle);
    }
    keyFromSeed(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyFromSeed(serializedOptions)));
        return new askar_shared_1.LocalKeyHandle(handle);
    }
    keyGenerate(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyGenerate(serializedOptions)));
        return new askar_shared_1.LocalKeyHandle((0, askar_shared_1.handleInvalidNullResponse)(handle));
    }
    keyGetAlgorithm(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyGetAlgorithm(serializedOptions)));
    }
    keyGetEphemeral(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyGetEphemeral(serializedOptions)));
    }
    keyGetJwkPublic(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyGetJwkPublic(serializedOptions)));
    }
    keyGetJwkSecret(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const buf = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyGetJwkSecret(serializedOptions)));
        return new Uint8Array(buf);
    }
    keyGetJwkThumbprint(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyGetJwkThumbprint(serializedOptions)));
    }
    keyGetPublicBytes(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const buf = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyGetPublicBytes(serializedOptions)));
        return new Uint8Array(buf);
    }
    keyGetSecretBytes(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const buf = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyGetSecretBytes(serializedOptions)));
        return new Uint8Array(buf);
    }
    keySignMessage(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const buf = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keySignMessage(serializedOptions)));
        return new Uint8Array(buf);
    }
    keyUnwrapKey(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.keyUnwrapKey(serializedOptions)));
        return new askar_shared_1.LocalKeyHandle(handle);
    }
    keyVerifySignature(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const result = this.handleError(this.askar.keyVerifySignature(serializedOptions));
        return !!result;
    }
    keyWrapKey(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const ret = this.handleError(this.askar.keyWrapKey(serializedOptions));
        const { buffer, noncePos, tagPos } = (0, askar_shared_1.handleInvalidNullResponse)(ret);
        return new askar_shared_1.EncryptedBuffer({ tagPos, noncePos, buffer: new Uint8Array(buffer) });
    }
    scanFree(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        // null resopnse is expected as we're freeing the object
        this.handleError(this.askar.scanFree(serializedOptions));
    }
    async scanNext(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = await this.promisifyWithResponse((cb) => this.handleError(this.askar.scanNext(Object.assign({ cb }, serializedOptions))));
        return askar_shared_1.EntryListHandle.fromHandle(handle);
    }
    async scanStart(options) {
        const { category, storeHandle, limit, offset, profile, tagFilter, orderBy, descending } = (0, serialize_1.serializeArguments)(options);
        const handle = await this.promisifyWithResponse((cb) => this.handleError(this.askar.scanStart({
            cb,
            category,
            storeHandle,
            offset: offset || 0,
            limit: limit || -1,
            orderBy,
            descending,
            profile,
            tagFilter,
        })));
        return askar_shared_1.ScanHandle.fromHandle(handle);
    }
    sessionClose(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return this.promisify((cb) => this.handleError(this.askar.sessionClose(Object.assign({ cb }, serializedOptions))));
    }
    async sessionCount(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const response = await this.promisifyWithResponse((cb) => this.handleError(this.askar.sessionCount(Object.assign({ cb }, serializedOptions))));
        return (0, askar_shared_1.handleInvalidNullResponse)(response);
    }
    async sessionFetch(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = await this.promisifyWithResponse((cb) => this.handleError(this.askar.sessionFetch(Object.assign({ cb }, serializedOptions))));
        return askar_shared_1.EntryListHandle.fromHandle(handle);
    }
    async sessionFetchAll(options) {
        const { category, sessionHandle, forUpdate, limit, tagFilter, descending, orderBy } = (0, serialize_1.serializeArguments)(options);
        const handle = await this.promisifyWithResponse((cb) => this.handleError(this.askar.sessionFetchAll({
            cb,
            category,
            sessionHandle,
            forUpdate,
            limit: limit || -1,
            tagFilter,
            orderBy,
            descending,
        })));
        return askar_shared_1.EntryListHandle.fromHandle(handle);
    }
    async sessionFetchAllKeys(options) {
        const { sessionHandle, algorithm, forUpdate, limit, thumbprint, tagFilter } = (0, serialize_1.serializeArguments)(options);
        const handle = await this.promisifyWithResponse((cb) => this.handleError(this.askar.sessionFetchAllKeys({
            cb,
            sessionHandle,
            algorithm,
            forUpdate: forUpdate || -1,
            limit: limit || -1,
            thumbprint,
            tagFilter,
        })));
        return askar_shared_1.KeyEntryListHandle.fromHandle(handle);
    }
    async sessionFetchKey(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = await this.promisifyWithResponse((cb) => this.handleError(this.askar.sessionFetchKey(Object.assign({ cb }, serializedOptions))));
        return askar_shared_1.KeyEntryListHandle.fromHandle(handle);
    }
    sessionInsertKey(options) {
        const { sessionHandle, name, localKeyHandle, expiryMs, metadata, tags } = (0, serialize_1.serializeArguments)(options);
        return this.promisify((cb) => this.handleError(this.askar.sessionInsertKey({
            cb,
            sessionHandle,
            name,
            localKeyHandle,
            expiryMs: expiryMs || -1,
            metadata,
            tags,
        })));
    }
    async sessionRemoveAll(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const response = await this.promisifyWithResponse((cb) => this.handleError(this.askar.sessionRemoveAll(Object.assign({ cb }, serializedOptions))));
        return (0, askar_shared_1.handleInvalidNullResponse)(response);
    }
    sessionRemoveKey(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return this.promisify((cb) => this.handleError(this.askar.sessionRemoveKey(Object.assign({ cb }, serializedOptions))));
    }
    async sessionStart(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = await this.promisifyWithResponse((cb) => this.handleError(this.askar.sessionStart(Object.assign({ cb }, serializedOptions))));
        return askar_shared_1.SessionHandle.fromHandle(handle);
    }
    sessionUpdate(options) {
        const { category, name, operation, sessionHandle, expiryMs, tags, value } = (0, serialize_1.serializeArguments)(options);
        return this.promisify((cb) => this.handleError(this.askar.sessionUpdate({
            cb,
            category,
            name,
            operation,
            sessionHandle,
            expiryMs: expiryMs || -1,
            tags,
            value,
        })));
    }
    sessionUpdateKey(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return this.promisify((cb) => this.handleError(this.askar.sessionUpdateKey(Object.assign({ cb }, serializedOptions))));
    }
    storeClose(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return this.promisify((cb) => this.handleError(this.askar.storeClose(Object.assign({ cb }, serializedOptions))));
    }
    storeCopyTo(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return this.promisify((cb) => this.handleError(this.askar.storeCopyTo(Object.assign({ cb }, serializedOptions))));
    }
    async storeCreateProfile(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const response = await this.promisifyWithResponse((cb) => this.handleError(this.askar.storeCreateProfile(Object.assign({ cb }, serializedOptions))));
        return (0, askar_shared_1.handleInvalidNullResponse)(response);
    }
    storeGenerateRawKey(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return (0, askar_shared_1.handleInvalidNullResponse)(this.handleError(this.askar.storeGenerateRawKey(serializedOptions)));
    }
    async storeGetProfileName(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const response = await this.promisifyWithResponse((cb) => this.handleError(this.askar.storeGetProfileName(Object.assign({ cb }, serializedOptions))));
        return (0, askar_shared_1.handleInvalidNullResponse)(response);
    }
    async storeGetDefaultProfile(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const response = await this.promisifyWithResponse((cb) => this.handleError(this.askar.storeGetDefaultProfile(Object.assign({ cb }, serializedOptions))));
        return (0, askar_shared_1.handleInvalidNullResponse)(response);
    }
    async storeListProfiles(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const stringListHandle = (0, askar_shared_1.handleInvalidNullResponse)(await this.promisifyWithResponse((cb) => this.handleError(this.askar.storeListProfiles(Object.assign({ cb }, serializedOptions)))));
        const ret = [];
        const count = this.handleError(this.askar.stringListCount({ stringListHandle }));
        for (let index = 0; index < count; index++) {
            ret.push(this.handleError(this.askar.stringListGetItem({ stringListHandle, index })));
        }
        this.handleError(this.askar.stringListFree({ stringListHandle }));
        return ret;
    }
    async storeOpen(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = await this.promisifyWithResponse((cb) => this.handleError(this.askar.storeOpen(Object.assign({ cb }, serializedOptions))));
        return askar_shared_1.StoreHandle.fromHandle(handle);
    }
    async storeProvision(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const handle = await this.promisifyWithResponse((cb) => this.handleError(this.askar.storeProvision(Object.assign({ cb }, serializedOptions))));
        return askar_shared_1.StoreHandle.fromHandle(handle);
    }
    storeRekey(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return this.promisify((cb) => this.handleError(this.askar.storeRekey(Object.assign({ cb }, serializedOptions))));
    }
    async storeRemove(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const response = await this.promisifyWithResponse((cb) => this.handleError(this.askar.storeRemove(Object.assign({ cb }, serializedOptions))));
        return (0, askar_shared_1.handleInvalidNullResponse)(response);
    }
    async storeRemoveProfile(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const response = await this.promisifyWithResponse((cb) => this.handleError(this.askar.storeRemoveProfile(Object.assign({ cb }, serializedOptions))));
        return (0, askar_shared_1.handleInvalidNullResponse)(response);
    }
    async storeSetDefaultProfile(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        const response = await this.promisifyWithResponse((cb) => this.handleError(this.askar.storeSetDefaultProfile(Object.assign({ cb }, serializedOptions))));
        (0, askar_shared_1.handleInvalidNullResponse)(response);
    }
    async migrateIndySdk(options) {
        const serializedOptions = (0, serialize_1.serializeArguments)(options);
        return this.promisify((cb) => this.handleError(this.askar.migrateIndySdk(Object.assign({ cb }, serializedOptions))));
    }
}
exports.ReactNativeAskar = ReactNativeAskar;
//# sourceMappingURL=ReactNativeAskar.js.map